

def test_calendar_between_allows_tuple(todo):
    pass

def test_calendar_between_allows_float(todo):
    pass

def test_calendar_between_allows_date(todo):
    pass

def test_calendar_between_allows_datetime(todo):
    pass
